"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMatchNamespacePatternItem = exports.isMatchDirPatternItem = exports.isMatchPatternItem = exports.isMatchPattern = exports.sortMatchesBySpecificity = exports.EXCLUSION_SIGN = exports.MATCH_ALL_ITEM = exports.PATTERNS_DELIMITER = void 0;
const string_strip_trailing_char_1 = require("@teambit/string.strip-trailing-char");
const path_is_path_inside_1 = require("@teambit/path.is-path-inside");
const ramda_1 = require("ramda");
const lodash_1 = __importDefault(require("lodash"));
exports.PATTERNS_DELIMITER = ',';
exports.MATCH_ALL_ITEM = '*';
exports.EXCLUSION_SIGN = '!';
function sortMatchesBySpecificity(matches) {
    const sortedMatches = ramda_1.sortBy(ramda_1.prop('specificity'), matches).reverse();
    return sortedMatches;
}
exports.sortMatchesBySpecificity = sortMatchesBySpecificity;
function isMatchPattern(rootDir, componentName, pattern) {
    const patternItems = pattern.split(exports.PATTERNS_DELIMITER);
    const matches = patternItems.map((item) => isMatchPatternItem(rootDir, componentName, item));
    const defaultVal = {
        match: false,
        excluded: false,
        specificity: -1,
    };
    const maxMatch = lodash_1.default.maxBy(matches, (match) => match.specificity) || defaultVal;
    const excluded = ramda_1.any((match) => match.excluded, matches);
    return {
        match: maxMatch.match,
        maxSpecificity: maxMatch.specificity,
        excluded,
    };
}
exports.isMatchPattern = isMatchPattern;
function isMatchPatternItem(rootDir, componentName, patternItem) {
    const patternItemTrimmed = patternItem.trim();
    // Special case for * (match all)
    // We want to mark it with match true but the smallest specificity
    if (patternItemTrimmed === exports.MATCH_ALL_ITEM) {
        return {
            match: true,
            specificity: 0,
            excluded: false,
        };
    }
    const { excluded, patternItemWithoutExcludeSign } = parseExclusion(patternItemTrimmed);
    if (isDirItem(patternItemWithoutExcludeSign)) {
        return Object.assign(Object.assign({}, isMatchDirPatternItem(rootDir, patternItemWithoutExcludeSign)), { excluded });
    }
    return Object.assign(Object.assign({}, isMatchNamespacePatternItem(componentName, patternItemWithoutExcludeSign)), { excluded });
}
exports.isMatchPatternItem = isMatchPatternItem;
function isMatchDirPatternItem(rootDir, patternItem) {
    // remove trailing / (will work for windows as well since the workspace.json always contain linux format)
    const patternItemStriped = string_strip_trailing_char_1.stripTrailingChar(patternItem, '/').trim();
    if (path_is_path_inside_1.isPathInside(rootDir, patternItemStriped)) {
        return {
            match: true,
            specificity: patternItemStriped.split('/').length,
        };
    }
    return {
        match: false,
        specificity: -1,
    };
}
exports.isMatchDirPatternItem = isMatchDirPatternItem;
function isMatchNamespacePatternItem(componentName, patternItem) {
    // remove trailing / (will work for windows as well since the workspace.json always contain linux format)
    const withoutBrackets = patternItem.replace('{', '').replace('}', '').trim();
    const patternItemStriped = string_strip_trailing_char_1.stripTrailingChar(withoutBrackets, '/').trim();
    let match = true;
    let specificity = 0;
    const splittedComp = componentName.split('/');
    const splittedPattern = patternItemStriped.split('/');
    if (splittedPattern.length > splittedComp.length) {
        return {
            match: false,
            specificity: -1,
        };
    }
    if (splittedPattern.length < splittedComp.length && splittedPattern[splittedPattern.length - 1] !== '*') {
        return {
            match: false,
            specificity: -1,
        };
    }
    splittedPattern.forEach((patternElement, index) => {
        const componentElement = splittedComp[index];
        if (patternElement === '*') {
            specificity += index / 10;
            return;
        }
        if (patternElement === componentElement) {
            specificity += 1;
            return;
        }
        match = false;
    });
    return {
        match,
        specificity: match ? specificity : -1,
    };
}
exports.isMatchNamespacePatternItem = isMatchNamespacePatternItem;
function isDirItem(patternItem) {
    return !patternItem.startsWith('{');
}
function parseExclusion(patternItem) {
    if (patternItem.startsWith(exports.EXCLUSION_SIGN)) {
        return {
            excluded: true,
            patternItemWithoutExcludeSign: patternItem.replace('!', ''),
        };
    }
    return {
        excluded: false,
        patternItemWithoutExcludeSign: patternItem,
    };
}
//# sourceMappingURL=match-pattern.js.map